@@ -1,3 +1,5 @@
+import uuid
+import decimal
 from enum import Enum
 from typing import Dict, Any, List, Optional, TypeVar, Union, Type, Set, Generic
 import six
@@ -6,6 +8,7 @@ import copy
 import pytz
 import json
 import datetime
+import jdatetime
 # noinspection PyPackageRequirements
 from dateutil import parser
 
@@ -52,7 +55,8 @@ class Field(JsonSerializable):
                 raise Exception('Cannot access field without instance')
             instance._data[self.serialized_name] = value
 
-    def __init__(self, field_type: FieldType,
+    def __init__(self,
+                 field_type: FieldType,
                  serialized_name: Optional[str] = None,
                  null: bool = True,
                  default_value: Optional[Any] = None):
@@ -105,7 +109,7 @@ class FloatField(Field):
                 value = float(value)
             except ValueError:
                 pass
-        if value is not None and isinstance(value, int):
+        if value is not None and isinstance(value, int) and self.accepts_int:
             value = float(value)
         if value is not None and not isinstance(value, float):
             raise TypeError(instance, self.name, float, value)
@@ -118,12 +122,14 @@ class FloatField(Field):
     def __init__(self, serialized_name: Optional[str] = None, null: bool = True,
                  default_value: Optional[float] = None,
                  accepts_string: bool = False,
+                 accepts_int: bool = True,
                  min_value: Optional[float] = None, max_value: Optional[float] = None):
         super(FloatField, self).__init__(field_type=FieldType.FLOAT,
                                          serialized_name=serialized_name,
                                          null=null,
                                          default_value=default_value)
         self.accepts_string = accepts_string
+        self.accepts_int = accepts_int
         assert default_value is None or isinstance(default_value, float)
         assert min_value is None or isinstance(min_value, float)
         assert max_value is None or isinstance(max_value, float)
@@ -441,6 +447,60 @@ class DateTimeField(Field):
         assert default_value is None or isinstance(default_value, datetime.datetime)
 
 
+class JDateField(Field):
+    def get_json_formatted_value(self, value):
+        return jdatetime.date.strftime(value, self.date_format)
+
+    def __set__(self, instance, value, test: bool = False):
+        if value is not None and isinstance(value, str):
+            try:
+                value = jdatetime.datetime.strptime(value, self.date_format).date()
+            except Exception as ex:
+                raise Exception('Error parsing date ' + str(value) + ' with given format ' +
+                                str(self.date_format) + ', error: ' + str(ex))
+        if value is not None and not isinstance(value, jdatetime.date):
+            raise TypeError(instance, self.name, jdatetime.date, value)
+        super().__set__(instance, value)
+
+    def __init__(self, date_format: str = '%Y-%m-%d', serialized_name: Optional[str] = None, null: bool = True):
+        super(JDateField, self).__init__(field_type=FieldType.STRING, serialized_name=serialized_name, null=null)
+        self.date_format = date_format
+
+
+class JDateTimeField(Field):
+    def get_json_formatted_value(self, value):
+        if value is None:
+            return None
+        return jdatetime.datetime.strftime(value, self.datetime_format)
+
+    def __set__(self, instance, value, test: bool = False):
+        if value is not None and isinstance(value, str):
+            try:
+                dt = jdatetime.datetime.strptime(value, self.datetime_format)
+            except ValueError:
+                raise Exception(
+                    'Error parsing date ' + str(value) + ' with given format ' + str(self.datetime_format))
+            value = pytz.timezone(self.datetime_timezone).localize(dt) \
+                if dt.tzinfo is None or dt.tzinfo.utcoffset(dt) is None else dt
+        if value is not None and not isinstance(value, jdatetime.datetime):
+            raise TypeError(instance, self.name, jdatetime.datetime, value)
+        super().__set__(instance, value, test)
+
+    def __init__(self,
+                 datetime_format: str = '%Y-%m-%d %H:%M:%S',
+                 datetime_timezone: str = 'UTC',
+                 serialized_name: Optional[str] = None,
+                 null: bool = True,
+                 default_value: Optional[datetime.datetime] = None):
+        super(JDateTimeField, self).__init__(field_type=FieldType.STRING,
+                                             serialized_name=serialized_name,
+                                             null=null,
+                                             default_value=default_value)
+        self.datetime_format = datetime_format
+        self.datetime_timezone = datetime_timezone
+        assert default_value is None or isinstance(default_value, jdatetime.datetime)
+
+
 class TimestampSecondsField(Field):
     def get_json_formatted_value(self, value):
         if value is None:
@@ -497,6 +557,56 @@ class TimestampMillisecondsField(Field):
         assert default_value is None or isinstance(default_value, datetime.datetime)
 
 
+class DecimalField(Field):
+    def __set__(self, instance, value, test: bool = False):
+        if value is not None and isinstance(value, str) and self.accepts_string:
+            value = decimal.Decimal(value)
+        if value is not None and not isinstance(value, decimal.Decimal):
+            raise TypeError(instance, self.name, decimal.Decimal, value)
+        super().__set__(instance, value, test)
+
+    def __init__(self, serialized_name: Optional[str] = None,
+                 null: bool = True,
+                 default_value: Optional[decimal.Decimal] = None,
+                 accepts_string: bool = False,
+                 ):
+        super(DecimalField, self).__init__(field_type=FieldType.STRING,
+                                           serialized_name=serialized_name,
+                                           null=null,
+                                           default_value=default_value)
+        self.accepts_string = accepts_string
+        assert default_value is None or isinstance(default_value, str)
+
+
+class UUIDField(Field):
+    def __set__(self, instance, value, test: bool = False):
+        if value is not None and isinstance(value, str):
+            try:
+                value = uuid.UUID(value, version=self.version)
+            except ValueError:
+                raise Exception(f'Provided string value {value} is not a valid UUID of chosen version {self.version}')
+        if value is not None and not isinstance(value, uuid.UUID):
+            raise TypeError(instance, self.name, uuid.UUID, value)
+        super().__set__(instance, value, test)
+
+    def __init__(self, serialized_name: Optional[str] = None,
+                 null: bool = True,
+                 default_value: Optional[uuid.UUID] = None,
+                 accepts_string: bool = False,
+                 version: int = 4,
+                 ):
+        super(UUIDField, self).__init__(field_type=FieldType.STRING,
+                                        serialized_name=serialized_name,
+                                        null=null,
+                                        default_value=default_value)
+        self.accepts_string = accepts_string
+        self.version = version
+        assert default_value is None or isinstance(default_value, str)
+
+
+
+
+
 class JsonField(Field):
     def __set__(self, instance, value, test: bool = False):
         if value is not None and not isinstance(value, dict):
